require "yaml"

class MetalAdventuresDiscord::RefsPlugin
  include Gem::Text # #levenshtein_distance/2

  def initialize(bot)
    @bot = bot
    @ref_cache = {}
    hook_ref!
  end

  REF_HOOK_SPECIAL = {
    "short" => "short",
    "court" => "short",
    "long"  => "long",
    "ref"   => "ref",
    "page"  => "ref",
    "full"  => "full",
    "tout"  => "full",
  }

  # !ref short blablabla
  # !ref full blablabla
  # !ref blablabla
  # event.message.reply! "Yo frèro", embed: { image: { url: event.user.avatar_url } }
  def hook_ref!
    @bot.message(with_text: /^!refs$/) do |event|
      event.message.reply Dir.glob("scenes/actions/*.yaml").map { _1.tr('_', ' ').split('/').last.split('.').first }.join(', ')
    end

    @bot.message(with_text: /!ref .+/) do |event|
      match = event.message.content.match(/\A!ref (?<total>(?<modifier>\w+ )?(?<nomodifier>.+))/)
      ref_name = match["total"]

      display_choice = nil
      modifier = match && match["modifier"].to_s.rstrip
      if REF_HOOK_SPECIAL[modifier]
        display_choice = modifier
        ref_name = match["nomodifier"]
      end

      ref_filename = "#{ref_name.downcase.tr(' ', '_')}.yaml"

      begin
        ref = load_ref(ref_filename)
        event.message.reply! "", embed: ref_to_embed(event, ref, display_choice)
      rescue StandardError => _e
        event.message.reply! "", embed: _err_invalid_ref_embed(ref_name)
      end
    end
  end

  def _err_invalid_ref_embed(ref)
    {
      color: 0xFF0000,
      title: "Error: invalid ref \"#{ref}\"",
      description: "!roll [full/short/ref/long] name of the page",
    }
  end

  def load_ref(filename)
    @ref_cache[filename] ||= YAML.load_file("scenes/actions/#{filename}")
  end

  DURATIONS = { "S" => "Simple", "C" => "Complexe", "G" => "Gratuite", nil => "Spéciale" }.freeze
  REFS = {
    "MJ1" => "Manuel du Joueur 1.0",
    "MJ" => "Manuel du Joueur 1.5",
    "MM" => "Manuel du Meneur",
    "PP" => "Prise et Profits",
    "FS" => "Fer et Sang",
    "PE" => "Pirates de l'Espace",
    "MA" => "Mutant Apocalypse",
  }
  def ref_to_embed(_event, ref, display_choice)
    description =
      case display_choice
      when "full"
        [ref['long'], "_#{ref['description']}_"].join("\n\n")
      when "short"
        ref['short']
      when "long"
        ref['long']
      when "ref"
        ref_manual, ref_page = ref['ref'].split(" ")
        "#{REFS[ref_manual]}, page #{ref_page}"
      else
        ref['short'] || ref['long'] || "Pas de données"
      end

    duration = DURATIONS[ref['duration']]
    with_infos = ["**Compétence**: #{ref['skill']}", "**Durée**: #{duration}", description].join("\n\n")

    {
      title: "#{ref['name']} (#{ref['ref']})",
      description: with_infos,
    }
  end
end
