module MetalAdventures::Scene::DSL
  class Scene
    # DSL
    # @example
    #
    #   Scene.new "coucou" do
    #     action "Machinerie" do
    #       option "Canaliser", duration: "C", skill: "Ingénierie"
    #     end
    #   end

    attr_accessor :name, :actions

    def initialize(name, &block)
      @name = name
      @actions = {}
      instance_eval(&block) if block_given?
    end

    def action(name, &block)
      @actions[name] = Action.new(name, &block)
    end

    class Action
      attr_accessor :name, :options

      def initialize(name, &block)
        @name = name
        @options = []
        instance_eval(&block) if block_given?
      end

      def option(name, duration: "C", skill: nil)
        @options << Option.new(name, duration: duration, skill: skill)
      end

      class Option
        attr_accessor :name, :duration, :skill

        def initialize(name, duration:, skill:)
          @name = name
          @duration = duration
          @skill = skill
        end
      end
    end
  end
end
